/*
 *	gen_sine.c
 *
 *	C Version 1.6 by Steve Sampson, Public Domain, 15 August 1988
 *
 *	This program is used to generate time domain sinewave data
 *	for fft.c.  If you want an opening target - negate the test frequency
 *
 *	Usage: gen_sine samples output
 */

#include <stdio.h>
#include <malloc.h>
#include <math.h>

#define	TWO_PI	((double)2.0 * 3.14159265358979324)
#define Chunk	((unsigned)(samples * sizeof(double)))

#ifdef	SYSV
#define	WMODE	"w"
#else
#define	WMODE	"wb"
#endif

static double	sample, freq, time, *real, *imag;
static int	loop, samples;
static void	err_report();
static FILE	*fp;


main(argc, argv)
int	argc;
char	*argv[];
{
	if (argc != 3)
		err_report(0);

	samples = abs(atoi(argv[1]));

	if ((real = (double *)malloc(Chunk)) == (double *)NULL)
		err_report(1);

	if ((imag = (double *)malloc(Chunk)) == (double *)NULL)
		err_report(2);

	printf("Input The Test Frequency (Hz) ? ");
	scanf("%lf", &freq);
	printf("Input The Sampling Frequency (Hz) ? ");
	scanf("%lf", &sample);
	sample = (double)1.0 / sample;

	time = (double)0.0;
	for (loop = 0; loop < samples; loop++)  {
		real[loop] =  sin(TWO_PI * freq * time);
		imag[loop] = -cos(TWO_PI * freq * time);
		time += sample;
	}

	if ((fp = fopen(argv[2], WMODE)) == (FILE *)NULL)
		err_report(3);

	fwrite((char *)real, sizeof(double), (unsigned)samples, fp);
	fwrite((char *)imag, sizeof(double), (unsigned)samples, fp);
	fclose(fp);

	putchar('\n');

	free((char *)real);
	free((char *)imag);

	exit(0);
}


static void err_report(n)
int	n;
{
	switch (n)  {
	case 0:
		fprintf(stderr,"Usage: gen_sine samples output_file\n");
		fprintf(stderr,"Where samples is a power of two\n");
		break;
	case 1:
		fprintf(stderr,"gen_sine: Out of memory getting real space\n");
		break;
	case 2:
		fprintf(stderr,"gen_sine: Out of memory getting imag space\n");
		free((char *)real);
		break;
	case 3:
		fprintf(stderr,"gen_sine: Unable to create write file\n");
	}

	exit(1);
}

/* EOF */
